@@ -1,21 +1,18 @@
 package Shell::POSIX::Select;
 
+our $VERSION = '0.03';
+
 # Tim Maher, tim@teachmeperl.com, yumpy@cpan.org
 # Fri May  2 10:29:25 PDT 2003
 # Mon May  5 10:51:49 PDT 2003
 
 # TO DO: portable-ize tput stuff
-# WHen lasting out of a nested loop, should reset do_prompt for outer one
-# Add logic to avoid looking for same failed match 2nd time, to save cycles
-# dump user's code-block with same line numbers shown in error messages for debugging ease
-# Add option to embolden menu numbers, to distinguish from choices that are also numbers
+# dump user's code-block with same line numbers shown in
+# error messages for debugging ease
+# Add option to embolden menu numbers, to distinguish from
+# choices that are also numbers
 # See documentation and copyright notice below =pod section below
 
-#+ Following used to harvest REs for use, without "use"-ing Regexp::Common
-#+	print STDOUT	$RE{balanced}{-parens=>'()'};
-#+	print STDERR	$RE{balanced}{-parens=>'{}'};
-#+ exit;
-
 
 # Not using Exporter.pm; doing typeglob-based exporting,
 # using adapted code from Damian's Switch.pm
@@ -30,13 +27,9 @@ our ( $DIRSEP, $sdump, $cdump, $script );
 our ( @ISA, @EXPORT, $PRODUCTION, $LOGGING, $PKG, $INSTALL_TESTING,$ON,$OFF, $BOLD, $SGR0, $COLS );
 
 BEGIN {
-# print "ENV reference is $ENV{Shell_POSIX_Select_reference}\n";
-# print "ENV testmode is $ENV{Shell_POSIX_Select_testmode}\n";
-# die;
 	$PKG  = __PACKAGE__ ;
 	$LOGGING = 0;
-	# Probably arrives during prompt, so nice to issue \n to get shell's prompt on next line
-# Revisit; what package should this be in?
+
 	$SIG{TERM}=$SIG{QUIT}=$SIG{INT}= sub {
 		warn  caller(1), "\n";
 		warn $OFF || $SGR0, "\n$0: killed by signal\n";	
@@ -53,17 +46,21 @@ use File::Spec::Functions (':ALL');
 use strict;
 # no strict 'refs';	# no problem now
 
-
 use File::Spec::Functions 0.7;
-use Filter::Simple;	# some bugs in this or its relatives, that can cause compilation errors here
+# some bugs in F::S or its relatives, that can cause compilation errors here
+use Filter::Simple;
+
+# Damian's been fixing bugs as I report them, so best to have recent version
+# This is the oldest version that I know works pretty well
 use Text::Balanced 1.89 qw(extract_variable extract_bracketed);
 
-# Damian has been fixing bugs as I report them, so it's best to have latest version
-# I've been testing with this as yet unrelased version
+# I've done most testing with this as yet unrelased version
 # use Text::Balanced 1.90 qw(extract_variable extract_bracketed);
+
 use Carp;
 
 # Why doesn't File:Spec just hand me the dir-separator char?
+# Sheesh, this should be a lot easier.
 ( $DIRSEP = catfile ( 1,2 ) ) =~ s/^1(.*)2$/$1/;
 
 $U_DEBUG=1;
@@ -99,11 +96,8 @@ $Shell::POSIX::Select::_arrows_prompt='>>';
 
 $U_WARN_default = 1;	# for enabling user-warnings for bad interactive input
 
-
 # $_import_called > 0 or import(); # ensure initialization of defaults
 
-our $VERSION = '0.02';
-
 my $subname=__PACKAGE__ ;	# for identifying messages from outside sub's
 
 my $select2foreach;
@@ -449,7 +443,6 @@ sub show_context {
 	show_subs( "Right is", $right, 0, 10);
 }
 
-	my $orig_string=$_;
 
 
 
@@ -469,9 +462,6 @@ sub matches2fields {
 
 
 
-#+	if ( $Shell::POSIX::Select::_Fudging and $two =~ /[\000..\025]/ ) {
-#+		$two = '1 ,2';
-#+	}
 
 	$debugging_code = "";
 	if ($U_DEBUG > 3) {
@@ -547,6 +537,8 @@ sub matches2fields {
 			$DEBUG > 1 and show_subs ("Pre-extract_variable 3\n");
 		my ( $loop_var2, @rest ) = extract_variable($loop_var);
 		if ( $loop_var2 ne $loop_var ) {
+			$DEBUG > 1 and
+				warn "$PKG: extracted var diff from parsed var: ",
 			$DEBUG > 0 and warn
 			  "$PKG: varname for select loop failed validation",
 			  " #$::_LOOP_COUNT: $loop_var\n";
@@ -575,7 +567,6 @@ sub enloop_codeblock {
 
 	$Shell::POSIX::Select::_ENLOOP_CALL_COUNT++;
 
-
 	my ( $decl, $loop_var, $values, $codestring, $dcode, $loopnum ) = @_;
 
 
@@ -595,21 +586,21 @@ sub enloop_codeblock {
 
 	# Now build the code for the user-prog to run
 	my @parts;
-	# Must start new scope first, so if user has LOOP: label before select, it applies
-	# to the whole encapsulated loop
-	# wrapper scope needed so user can LABEL: select(), and not try to label my label
+	# Start new scope first, so if user has LOOP: label before select, 
+	# it applies to the whole encapsulated loop
+	# wrapper scope needed so user can LABEL: select(), and not *my* label
 	push @parts, qq(
+	# Code generated by $PKG v$VERSION, by tim(AT)TeachMePerl.com
 	# NOTA BENE: Line 1 of this segment must start with {, so user can LABEL it
   { # **** NEW WRAPPER SCOPE FOR SELECTLOOP #$loopnum ****
 		\$${PKG}::DEBUG > 1 and $loopnum == 1 and
 			warn "LINE NUMBER FOR START OF USER CODE_BLOCK IS:  ", __LINE__, "\\n";
     _SEL_LOOP$loopnum: { # **** NEW SCOPE FOR SELECTLOOP #$loopnum ****
 	);
-# warn "LOGGING is now $LOGGING\n";
 	$LOGGING and (print PART1 $parts[0] or _DIE "failed to write to PART1\n");
 
 	push @parts, qq(
-	# critical for values's contents to be resolved in user's scope
+		# critical for values's contents to be resolved in user's scope
 		local \@$arrayname=$values;
 
 		local \$${PKG}::num_values=\@$arrayname;
@@ -621,10 +612,6 @@ sub enloop_codeblock {
 
 
 
-	# no strict 'refs';
-	# die;
-	# use strict 'refs';
-
 
 
 # CODING NOTES: don't use standalone word "select" in following,
@@ -684,10 +671,8 @@ sub enloop_codeblock {
 						exit 222;	# code for graceful, expected, early exit
 				}
           else {
-					# \$^W=0;
 						\$Eof=0;
             \$Reply = <STDIN>;
-					# \$^W=1;
 
             if ( !defined( \$Reply ) ) {
               defined "$BOLD" and "$BOLD" ne "" and print STDERR "$SGR0";
@@ -884,13 +869,13 @@ sub log_files {
 		# being converted to original data.	
 		#
 		$DEBUG > 1 and $LOGGING > 0 and warn "Opening log files\n";	
-		open LOG,	"> $dir${DIRSEP}SELECT_log" or _DIE "Open failed, $!\n";
-		open SOURCE,	"> $dir${DIRSEP}SELECT_source" or _DIE "Open failed, $!\n";	
-		open USERPROG,	"> $dir${DIRSEP}SELECT_user_program" or _DIE "Open failed, $!\n";	
-		open PART1,	"> $dir${DIRSEP}SELECT_part1" or _DIE "Open failed, $!\n";	
-		open PART2,	"> $dir${DIRSEP}SELECT_part2" or _DIE "Open failed, $!\n";
-		open PART3,	"> $dir${DIRSEP}SELECT_part3" or _DIE "Open failed, $!\n";	
-		open PART4,	"> $dir${DIRSEP}SELECT_part4" or _DIE "Open failed, $!\n";
+		open LOG,	"> $dir${DIRSEP}SELECT_log" or _DIE "Open LOG failed, $!\n";
+		open SOURCE,	"> $dir${DIRSEP}SELECT_source" or _DIE "Open SOURCE failed, $!\n";	
+		open USERPROG,	"> $dir${DIRSEP}SELECT_user_program" or _DIE "Open USERPROG failed, $!\n";	
+		open PART1,	"> $dir${DIRSEP}SELECT_part1" or _DIE "Open PART1 failed, $!\n";	
+		open PART2,	"> $dir${DIRSEP}SELECT_part2" or _DIE "Open PART2 failed, $!\n";
+		open PART3,	"> $dir${DIRSEP}SELECT_part3" or _DIE "Open PART3 failed, $!\n";	
+		open PART4,	"> $dir${DIRSEP}SELECT_part4" or _DIE "Open PART4 failed, $!\n";
 		$LOGGING++;	# to avoid 2nd invocation
 	}	
 	elsif ($LOGGING > 1) {	
@@ -956,8 +941,6 @@ for (my $i=0;  $i<@_; $i++) {
 	# accidentally mess with following hash-style options
 	$found==0 and last;
 }
-#+ %import and print "Hash is: ", %import, "\n";
-#+ %import and print "Detected imports: ",  keys %import, "\n";
 %import and export($user_pkg, keys %import);	# create aliases for user
 
 # following gets "attempt to delete unreferenced scalar"!
@@ -968,7 +951,6 @@ map { delete $_[$_] } values %import;	# but this works
 
 @_= grep defined,  @_;	# reset, to eliminate extracted imports
 # warn "Numvals in array is now ", scalar @_, "\n";
-#	$DEBUG > 4 and
 		# warnings sets user-program debugging level
 		# debug sets module's debuging level
 		my @legal_options = qw( style  prompt  testmode  warnings  debug logging );
@@ -985,7 +967,6 @@ map { delete $_[$_] } values %import;	# but this works
 		# timji: Loopify this section later, once it gets stable
 
 		# "logging" enables/disables logging of filter output to file 
-#+ warn "Checking logging variable\n";
 		$_ = $ENV{Shell_POSIX_Select_logging} || $options{logging};
 		if (defined) {
 			# unless ( is_unix() ) {
@@ -1121,14 +1102,6 @@ die 33;
 				}
 			}
 		# ENV variable overrides program spec
-#+		defined $ENV{Shell_POSIX_Select_style} and
-#+			$Shell::POSIX::Select::_style=$ENV{Shell_POSIX_Select_style} ;
-#+		defined $ENV{Shell_POSIX_Select_prompt} and
-#+			$Shell::POSIX::Select::_prompt=$ENV{Shell_POSIX_Select_prompt} ;
-#+		defined $ENV{Shell_POSIX_Select_debug} and
-#+			$Shell::POSIX::Select::_debug=$ENV{Shell_POSIX_Select_debug} ;
-#+		defined $ENV{Shell_POSIX_Select_testmode} and
-#+			$Shell::POSIX::Select::_testmode=$ENV{Shell_POSIX_Select_testmode} ;
 
 		( ! defined $Shell::POSIX::Select::_testmode or
 			$Shell::POSIX::Select::_testmode eq "" ) and
@@ -1140,36 +1113,37 @@ $DEBUG > 2 and warn "37 Testmode set to $Shell::POSIX::Select::_testmode\n";
 
 		$ENV{Shell_POSIX_Select_reference} and
 			$Shell::POSIX::Select::dump_data = 'Ref_Data';
+
+		# Don't assume /dev/tty will work on user's platform!
 		if ( $Shell::POSIX::Select::dump_data ) {
-			$Shell::POSIX::Select::_TTY=0;
-			open TTY,	"> /dev/tty" or _DIE "Open of TTY failed, $!\n";	
-			$Shell::POSIX::Select::_TTY=1;
+			#if ( ! $PRODUCTION ) {
+				$Shell::POSIX::Select::_TTY=0;
+				# What's the OS-portable equivalent of "/dev/tty" in the above?
+				if ( -c '/dev/tty' ) {
+					if ( open TTY,	'> /dev/tty' ) {
+						$Shell::POSIX::Select::_TTY=1;
+					}
+					else {
+							_WARN "Open of /dev/tty failed, $!\n";	
+					}
+				}
+			#}
 
-# What's the OS-portable equivalent of "/dev/tty" in the above?
-
-# print TTY "dump_data is $Shell::POSIX::Select::dump_data\n";
 			$sdump =
 			($Shell::POSIX::Select::dump_data =~ /[a-z]/i ? # Dir prefix, or nothing
 				$Shell::POSIX::Select::dump_data : '.') . $DIRSEP . "$script.sdump" .
 			($Shell::POSIX::Select::dump_data =~ /[a-z]/i ? # Dir prefix, or nothing
 				'_ref' : '') ;
 			($cdump = $sdump) =~ s/$script\.sdump/$script.cdump/;	# make code-dump name too
-# print TTY "ENV reference is $ENV{Shell_POSIX_Select_reference}\n";
-# print TTY "Shell::POSIX::Select::dump_data is $Shell::POSIX::Select::dump_data\n";
-# print TTY "Sdump is $sdump\n";
-# print TTY "Cdump is $cdump\n";
-# 			$Shell::POSIX::Select::_TTY and
-# 				print TTY "PREPARING DUMPS\n";
 			# Make reference copies of dumps for distribution, or test copies,
 			# depending on ENV{reference} set or testmode=make
 			close STDERR or
 				die "$PKG-END(): Failed to close 'STDERR', $!\n";
 			open STDERR, "> $sdump" or
 				die "$PKG-END(): Failed to open '$sdump' for writing, $!\n";
-# HERE2
+
 			open STDOUT, ">&STDERR" or
 				die "$PKG-END(): Failed to dup STDOUT to STDERR, $!\n";
-#			print TTY "${PKG}-END(): Succeeded in opening '$sdump' for writing\n";
 		}
 
 	( $ON , $OFF , $BOLD ,  $SGR0 , $COLS ) =
@@ -1183,10 +1157,7 @@ sub export {	# appropriated from Switch.pm
 	my $pkg = shift;
 	no strict 'refs';
 # All exports are scalard vars,  so strip sigils and poke in package name
-#+ warn "$subname received @_\n";
 	foreach ( map {  s/^\$//; $_ } @_ ) {	# must change $Reply to Reply, etc.
-#+ print "Aliaser for $pkg: \$_ is $_\n";
-#+ 			print "Working on Shell::POSIX::Select::$_\n";
 		*{"${pkg}::$_"} =
 			\${ "Shell::POSIX::Select::$_" };
 				# "Shell::POSIX::Select::$_";
@@ -1227,10 +1198,6 @@ sub hash_options {
 		}
 
 	}
-#+			warn "Original keys/values\n";
-#+ foreach (sort keys %options) { warn "$_/'$options{$_}'   "; }
-#+			warn "Final keys/values\n";
-#+ foreach (sort keys %options2) { warn "$_/'$options{$_}'   "; }
 
 	return %options2;
 }
@@ -1295,40 +1262,21 @@ sub display_control {
 }
 
 END { # END block
-# 		$Shell::POSIX::Select::_TTY and
-# 			print TTY "IN END BLOCK NOW\n"; 
-# 			$Shell::POSIX::Select::_TTY and
-# 		$ENV{Shell_POSIX_Select_reference} and print TTY "REF set\n";
-# 			$Shell::POSIX::Select::_TTY and
-#  		$ENV{Shell_POSIX_Select_dump_data} and print TTY "DS set\n";
-	# print "\$script is $script\n";
 	# sdump means screen-dump, cdump means code-dump
 	if ( $Shell::POSIX::Select::dump_data ) {
-# 		$Shell::POSIX::Select::_TTY and
-# 			print TTY "IN dump source BLOCK NOW\n"; 
 		if ( $ENV{Shell_POSIX_Select_reference} ) {
-# 			$Shell::POSIX::Select::_TTY and
-# 				print TTY "Reference variable IS defined\n";
-# 			$Shell::POSIX::Select::_TTY and
-# 			print TTY "Reference variable IS defined\n";	# double
-			# Make reference copies of code dumps, for distribution
-# 			$Shell::POSIX::Select::_TTY and
-# 				print TTY "Making reference code dumps to $cdump\n";
 		}
 		else {
-# 			$Shell::POSIX::Select::_TTY and
-# 				print "TTY Reference variable not defined\n";
 		}
-			my $pwd=`pwd`;
+			my $pwd=curdir();
 # 			$Shell::POSIX::Select::_TTY and
-# 				print TTY "Current dir is '$pwd'\n";
 			# dump filtered source, for reference or analysis
-# 			$Shell::POSIX::Select::_TTY and
-# 	 	  print TTY "Attempting open of $cdump for writing\n";
-			open SOURCE, "> $cdump" or
-			$Shell::POSIX::Select::_TTY and
- 				print TTY "$PKG-END(): Failed to open '$cdump' for writing, $!\n" and
-			die;
+			unless (open SOURCE, "> $cdump") {
+				$Shell::POSIX::Select::_TTY and
+				 print TTY "$PKG-END(): Failed to open '$cdump' for writing, $!\n" and
+				warn "$PKG-END(): Failed to open '$cdump' for writing, $!\n" ;
+				die;
+			}
 			defined $Shell::POSIX::Select::filter_output and
 				print SOURCE $Shell::POSIX::Select::filter_output or 
 				die "$PKG-END(): Failed to write to '$cdump', $!\n";
@@ -1366,7 +1314,7 @@ sub is_unix {
 
 =head1 NAME
 
-Shell::POSIX::Select - The POSIX Shell's C<select> loop for Perl
+Shell::POSIX::Select - The POSIX Shell's "select" loop for Perl
 
 =head1 PURPOSE
 
@@ -1375,21 +1323,27 @@ for Perl.
 That loop is unique in two ways: it's by far the friendliest feature of any UNIX shell,
 and it's the I<only> UNIX shell loop that's missing from the Perl language.  Until now!
 
+What's so great about this loop? It automates the generation of a numbered menu
+of choices, prompts for a choice, proofreads that choice and complains if it's invalid
+(at least in this enhanced implementation), and executes a code-block with a variable
+set to the chosen value.  That saves a lot of coding for interactive programs --
+especially if the menu consists of many values!
+
+The benefit of bringing this loop to Perl is that it obviates the
+need for future programmers
+to reinvent the I<Choose-From-A-Menu> wheel.
+
 =head1 SYNOPSIS
 
-NOTE: In the following, the enclosing square brackets I<(not typed)> identify optional elements, and vertical bars indicate mutually-exclusive choices:
-
+=for comment
+Resist temptation to add more spaces in line below; they cause bad wrapping for text-only document version
+ 
 B<select>
 [ [ my | local | our ] scalar_var ]
-B<(> [ LIST ] B<)>
-B<{> [ CODE ] B<}>
+B<(> [LIST] B<)>
+B<{> [CODE] B<}>
 
-=for comment
-C<select>
-B<[>B<[>C<my>|C<local>|c<our><]>
-C<scalar_var>B<]>
-( B<[>C<LIST>B<]> )
-{ B<[>C<CODE>B<]> } 
+In the above, the enclosing square brackets I<(not typed)> identify optional elements, and vertical bars indicate mutually-exclusive choices:
 
 The required elements are the keyword C<select>,
 the I<parentheses>, and the I<curly braces>.
@@ -1399,8 +1353,7 @@ See L<"SYNTAX"> for details.
 
 NOTE: All non-trivial programming examples shown in this document are
 distributed with this module, in the B<Scripts> directory.
-See L<"ADDITIONAL EXAMPLES">
-for more interesting uses of C<select>.
+L<"ADDITIONAL EXAMPLES">, covering more features, are shown below.
 
 =head2 ship2me.plx
 
@@ -1452,43 +1405,13 @@ B<Screen>
     You chose: FedEx
     Your order has been processed.  Thanks for your business!
 
-=head1 BENEFITS
-
-What's so great about this loop? It automates the generation of a numbered menu
-of choices, prompts for a choice, proofreads that choice and complains if it's invalid
-(at least in this enhanced implementation), and executes a code-block with a variable
-set to the chosen value.  That saves a lot of coding for interactive programs --
-especially if the menu consists of many values!
-
-The benefit of bringing this loop to Perl is that it obviates the
-need for future programmers
-to reinvent the I<Choose-From-A-Menu> wheel.
 
 =head1 SYNTAX
 
-=begin comment
-
-NOTE: Enclosing square brackets I<(never typed!)> identify optional elements in the following line, and vertical bars indicate mutually-exclusive choices:
-
-C<select>
-B<[>B<[>C<my>|C<local>|c<our><]>
-C<scalar_var>B<]>
-( B<[>C<LIST>B<]> )
-{ B<[>C<CODE>B<]> } 
-
-The required elements are the keyword C<select>,
-the I<parentheses>, and the I<curly braces>, but
-in most applications the
-programmer will want to supply one
-or more of the optional portions.
-
-=end comment
+=head2 Loop Structure
 
 Supported invocation formats include the following:
 
-=begin man
-.nf
-\f(CW
  use Shell::POSIX::Select ;
 
  select                 ()      { }         # Form 0
@@ -1499,37 +1422,6 @@ Supported invocation formats include the following:
  select our     $var    (LIST)  { CODE }    # Form 5
  select local   $var    (LIST)  { CODE }    # Form 6
 
-=end man
-
-=begin text
-
- use Shell::POSIX::Select ;
-
- select                 ()      { }         # Form 0
- select                 ()      { CODE }    # Form 1
- select                 (LIST)  { CODE }    # Form 2
- select         $var    (LIST)  { CODE }    # Form 3
- select my      $var    (LIST)  { CODE }    # Form 4
- select our     $var    (LIST)  { CODE }    # Form 5
- select local   $var    (LIST)  { CODE }    # Form 6
-
-=end text
-
-=begin html
-
-<PRE>
-<TT>
- select                 ()      { }         <I># Form 0</I>
- select                 ()      { CODE }    <I># Form 1</I>
- select                 (LIST)  { CODE }    <I># Form 2</I>
- select         $var    (LIST)  { CODE }    <I># Form 3</I>
- select my      $var    (LIST)  { CODE }    <I># Form 4</I>
- select our     $var    (LIST)  { CODE }    <I># Form 5</I>
- select local   $var    (LIST)  { CODE }    <I># Form 6</I>
-</TT>
-</PRE>
-
-=end html
 
 If the loop variable is omitted (as in I<Forms> I<0>, I<1> and I<2> above),
 it defaults to C<$_>, C<local>ized to the loop's scope.
@@ -1553,7 +1445,7 @@ The only form that's I<not> allowed is one that specifies the loop-variable's de
 See L<"SCOPING ISSUES"> for full details about the implications
 of different types of declarations for the loop variable.
 
-=head2 $Reply
+=head2 The $Reply Variable
 
 When the interactive user responds to the C<select> loop's prompt
 with a valid input (i.e., a number in the correct range),
@@ -1649,19 +1541,6 @@ when a terminal user submits EOF as input to a nested C<select> loop.
 In such cases, experience shows it's critical to reissue the
 menu of the outer loop before accepting any more input.
 
-=begin comment
-
-The response additionally appears in B<$ENV{REPLY}>, which warrants some discussion.
-In the Shells, the automatic exporting of that variable is adjustable via
-the B<set -a> or B<set -o allexport> option settings,
-but given that Perl has no such mechanism for automatically
-placing variables into the environment,
-a decision was made to have the module do this for the user.
-(This could become a configurable option
-in subsequent versions, if anybody cares).
-
-=end comment
-
 =head1 SCOPING ISSUES
 
 If the loop variable is named and provided with a I<declarator> (C<my>, C<our>, or C<local>),
@@ -1724,7 +1603,7 @@ upon exit from a C<select> loop (see L<"Eof Detection">).
      '$Prompt',      # to customize per-menu prompt
      '$Heading',     # to customize per-menu heading
      '$Eof',         # T/F for Eof detection
- # Variables must come first, then key/value options
+  # Variables must come first, then key/value options
      prompt   => 'Enter number of choice:',  # or 'whatever:'
      style    => 'Bash',     # or 'Korn'
      warnings => 1,          # or 0
@@ -1733,7 +1612,7 @@ upon exit from a C<select> loop (see L<"Eof Detection">).
      testmode => <unset>,    # or 'make', or 'foreach'
  );
 
-I<NOTE:> The values shown for options are the defaults, except for C<testmode>, which has none.
+I<NOTE:> The values shown for options are the defaults, except for C<testmode>, which doesn't have one.
 
 =head2 Prompts
 
@@ -1866,14 +1745,14 @@ checking that the program is syntactically correct before any serious
 filtering has been applied (which can introduce syntax errors).
 This works because the two loops, in their I<full forms>, have identical syntax.
 
-Note that before you use the C<foreach> option, you I<must> fill in any
+Note that before you use C<< testmode => foreach >>, you I<must> fill in any
 missing parts that are required by C<foreach>.
 
 For instance,
 
 C<	select () {}> 
 
-I<	must be rewritten as follows to explicitly show "@ARGV" (assuming it's not in a subroutine) and "print">
+must be rewritten as follows, to explicitly show "@ARGV" (assuming it's not in a subroutine) and "print":
 
 C<	foreach (@ARGV) { print; }>
 
@@ -1884,9 +1763,7 @@ distributed with this module, in the B<Scripts> directory.
 See L<"ELEMENTARY EXAMPLES">
 for simpler uses of C<select>.
 
-=head2 SMALL PROGRAMS
-
-=head3 pick_file.plx
+=head2 pick_file.plx
 
 This program lets the user choose filenames to be sent to the output.
 It's sort of like an
@@ -1914,54 +1791,11 @@ B<Screen>
 
     request id is yumpy@guru+587
 
-=begin comment
-
-h=ead3 long_listem.plx
-
-This program gives the user a menu of filenames, and displays
-a long-listing of each chosen one.
-
- use Shell::POSIX::Select ;
-
- select my $var (@ARGV) { system "ls -ldi '$var'" } # Maximal typing
-
-     # OR, through use of defaults, the following is equivalent:
-
- select ( ) { system "ls -ldi '$_'" } # Using default var & LIST
-
-
-B<Screen>
-
- long_listem.plx  *.txt
-
- 1) memo1.txt   2) memo2.txt   3) memo3.txt
- 4) junk1.txt   5) junk2.txt   6) junk3.txt
-
- Enter number of choice: 4
-
- -rw-r--r--    1 yumpy   gurus   18929 Jan  1 16:41 junk1.txt
-
-h=ead3 browse_jpeg.plx
-
-Here's a simple yet highly useful script.   It displays a menu of all
-the JPEG files in the current directory, and then displays the chosen
-ones on-screen using a backgrounded image viewer.
-
-    use Shell::POSIX::Select ;
-
-    $viewer='/usr/X11R6/bin/xv';   # Popular Linux viewer
-    select ( <*.jpg> ) {
-        system "$viewer $_ &" ;
-    }
-
-=end comment
-
-=head3 browse_images.plx
+=head2 browse_images.plx
 
 Here's a simple yet highly useful script.   It displays a menu of all
 the image files in the current directory, and then displays the chosen
 ones on-screen using a backgrounded image viewer.
-
 It uses Perl's C<grep> to filter-out filenames that don't
 end in the desired extensions.
 
@@ -1973,24 +1807,7 @@ end in the desired extensions.
         system "$viewer $_ &" ;     # run viewer in background
     }
 
-=begin comment
-
-h=ead3 send_spam.plx
-
-In the next example, the user selects an email address from a numbered menu of choices,
-and then mail is sent to that address.
-(Note that I'm not I<condoning> the behavior illustrated in the example;
-on the contrary, I'm promoting an interactive approach in order to I<slow them down>!)
-
- use Shell::POSIX::Select ( prompt=>'Enter number of spamee:' ) ;
-
- select $spamee ( Get_all_Email_Addresses_in_Universe() ) {
-         send_spam ( $spamee, '*! Make Money Fast !*' ) ;
- }
-
-=end comment
-
-=head3 perl_man.plx
+=head2 perl_man.plx
 
 Back in the olden days, we only had one Perl man-page. It was
 voluminous, but at least you knew what argument to give the B<man>
@@ -2029,12 +1846,10 @@ I<(This large menu spans multiple screens, but all parts can be accessed
         release and the 5.7.1 release.
  . . .
 
-=head2 MEDIUM PROGRAMS
-
-=head3 pick.plx
+=head2 pick.plx
 
 This more general C<pick>-ing program lets the user make selections
-from arguments, if they're present, or else input, in the spirit of Perl's
+from I<arguments>, if they're present, or else I<input>, in the spirit of Perl's
 C<-n> invocation option and C<< <> >> input operator.
 
  use Shell::POSIX::Select ;
@@ -2048,7 +1863,8 @@ C<-n> invocation option and C<< <> >> input operator.
          chomp @choices ;
          # STDIN already returned EOF, so must reopen
          # for terminal before menu interaction
-         open STDIN, "/dev/tty"  or  die ;  # UNIX example
+         open STDIN, "/dev/tty"  or
+             die "$0: Failed to open STDIN, $!" ;  # UNIX example
      }
  }
  select ( @choices ) { }   # prints selections to output
@@ -2057,7 +1873,7 @@ B<Sample invocations (UNIX-like system)>
 
     lp `pick *.txt`    # same output as shown for "pick_file"
 
-    find . -name '*.plx' -print | pick | lp
+    find . -name '*.plx' -print | pick | xargs lp  # includes sub-dirs
 
     who |
         awk '{ print $1 }' |        # isolate user names
@@ -2065,7 +1881,7 @@ B<Sample invocations (UNIX-like system)>
                 Mail -s 'Promote these people!'  boss
 
 
-=head3 delete_file.plx
+=head2 delete_file.plx
 
 In this program, the user selects a filename 
 to be deleted.  The outer loop is used to refresh the list,
@@ -2079,8 +1895,8 @@ necessary.
  ) ;
 
  OUTER:
-     while ( @files=<*.py> ) {  # loop exited from within
-         select ( @files ) { 
+     while ( @files=<*.py> ) { # collect serpentine files
+         select ( @files ) {   # prompt for deletions
              print STDERR  "Really delete $_? [y/n]: " ;
              my $answer = <STDIN> ;     # ^D sets $Eof below
              defined $answer  or  last OUTER ;  # exit on ^D
@@ -2089,7 +1905,7 @@ necessary.
          $Eof and last;
  }
 
-=head3 lc_filename.plx
+=head2 lc_filename.plx
 
 This example shows the benefit of importing C<$Eof>, 
 so the outer loop can be exited when the user supplies
@@ -2111,7 +1927,7 @@ as I<TRUE>, triggering the exit from the C<while> loop.
  );
 
  # Rename selected files from current dir to lowercase
- while ( @files=<*[A-Z]*> ) {    # restarts select to get updated menu
+ while ( @files=<*[A-Z]*> ) {   # refreshes select's menu
      select ( @files ) { # skip fully lower-case names
          if (rename $_, "\L$_") {
              last ;
@@ -2120,7 +1936,7 @@ as I<TRUE>, triggering the exit from the C<while> loop.
              warn "$0: rename failed for $_: $!\n";
          }
      }
-     $Eof  and  last ;   # Handle <^D> to menu prompt
+     $Eof  and  last ;   # Handle ^D to menu prompt
  }
 
 B<Screen>
@@ -2133,7 +1949,7 @@ B<Screen>
  1) Zeke.memo
  Enter number (^D to exit): ^D
 
-=head3 order.plx
+=head2 order.plx
 
 This program sets a custom prompt and heading for each of
 its two loops, and shows the use of a label on the outer loop.
@@ -2164,7 +1980,7 @@ B<Screen>
 
  Size Menu:
  1) L   2) XL
- Choose Size: <^D>       (changed my mind about the quantity)
+ Choose Size: ^D       (changed my mind about the quantity)
 
  Quantity Menu:
  1)  1    2)  2    3)  3    4)  4
@@ -2175,9 +1991,7 @@ B<Screen>
  Choose Size: 2
  You chose 2 units of size XL
 
-=head2 LARGE PROGRAMS
-
-=head3 browse_records.plx
+=head2 browse_records.plx
 
 This program shows how you can implement a "record browser",
 that builds a menu from the designated field of each record, and then
@@ -2224,7 +2038,7 @@ B<Screen>
 
  Enter number of choice: ^D
 
-=head3 menu_ls.plx
+=head2 menu_ls.plx
 
 This program shows a prototype for a menu-oriented front end
 to a UNIX command, that prompts the user for command-option choices,
@@ -2342,12 +2156,6 @@ can actually break your program:
  # { ; }
  select (@ARGV) { ; }
 
-=for comment
-That's because current versions of Filter::Simple
-perform substitutions for "select (...) {...}" even
-in the I<commented-out lines> of your program, which leads this module to
-introduce syntax errors into your program.
-
 A future version of Filter::Simple
 (or more precisely Text::Balanced, on which on which it depends)
 may correct this problem.
@@ -2398,19 +2206,6 @@ versions of that module
 have corrected this problem, 
 so download the latest version if you must use such names.
 
-=begin comment
-
-h=ead2 Solaris "Killed" Bug
-
-The module works properly under B<Solaris 9> (B<SunOS 5.9>),
-except for some reason it always prints C<Killed> before printing the
-expected output.  Anybody have a clue about this?
-
-(All the development occurred under Linux, where this symptom
-doesn't occur.)
-
-=end comment
-
 =head2 Please Report Bugs!
 
 This is a non-trivial program, that does some fairly complex parsing
@@ -2476,6 +2271,18 @@ See L<"IMPORTS AND OPTIONS"> for details.
  yumpy@cpan.org
  http://www.teachmeperl.com
 
+=begin html
+
+ -
+
+=end html
+
+=begin HTML
+
+ +
+
+=end HTML
+
 =head1 ACKNOWLEDGEMENTS
 
 I probably never would have even attempted to write this module
@@ -2510,6 +2317,10 @@ L<http://teachmeperl.com/mp4sh.html>) in early fall, 2003.
 B<perldoc -f select>, which has nothing to do with this module
 (the names just happen to match up).
 
+=head1 VERSION
+
+ This document describes version 0.03.
+
 =head1 LICENSE
 
 Copyright (C) 2002-2003, Timothy F. Maher.  All rights reserved. 
